@@ -16,7 +16,7 @@ test('20 times GET with pipelining 10', async (t) => {
 
   let count = 0
   let countGreaterThanOne = false
-  const server = createServer((req, res) => {
+  const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
     count++
     setTimeout(function () {
       countGreaterThanOne = countGreaterThanOne || count > 1
@@ -83,7 +83,7 @@ test('A client should enqueue as much as twice its pipelining factor', async (t)
 
   let count = 0
   let countGreaterThanOne = false
-  const server = createServer((req, res) => {
+  const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
     count++
     t.ok(count <= 5)
     setTimeout(function () {
@@ -137,7 +137,7 @@ test('pipeline 1 is 1 active request', async (t) => {
   t = tspl(t, { plan: 9 })
 
   let res2
-  const server = createServer((req, res) => {
+  const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
     res.write('asd')
     res2 = res
   })
@@ -185,7 +185,7 @@ test('pipelined chunked POST stream', async (t) => {
   let a = 0
   let b = 0
 
-  const server = createServer((req, res) => {
+  const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
     req.on('data', chunk => {
       // Make sure a and b don't interleave.
       t.ok(a === 9 || b === 0)
@@ -254,7 +254,7 @@ test('pipelined chunked POST iterator', async (t) => {
   let a = 0
   let b = 0
 
-  const server = createServer((req, res) => {
+  const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
     req.on('data', chunk => {
       // Make sure a and b don't interleave.
       t.ok(a === 9 || b === 0)
@@ -322,7 +322,7 @@ function errordInflightPost (bodyType) {
     t = tspl(t, { plan: 6 })
 
     let serverRes
-    const server = createServer()
+    const server = createServer({ joinDuplicateHeaders: true })
     server.on('request', (req, res) => {
       serverRes = res
       res.write('asd')
@@ -380,7 +380,7 @@ errordInflightPost(consts.ASYNC_ITERATOR)
 test('pipelining non-idempotent', async (t) => {
   t = tspl(t, { plan: 4 })
 
-  const server = createServer()
+  const server = createServer({ joinDuplicateHeaders: true })
   server.on('request', (req, res) => {
     setTimeout(() => {
       res.end('asd')
@@ -426,7 +426,7 @@ function pipeliningNonIdempotentWithBody (bodyType) {
   test(`pipelining non-idempotent w body ${bodyType}`, async (t) => {
     t = tspl(t, { plan: 4 })
 
-    const server = createServer()
+    const server = createServer({ joinDuplicateHeaders: true })
     server.on('request', (req, res) => {
       setImmediate(() => {
         res.end('asd')
@@ -489,7 +489,7 @@ function pipeliningHeadBusy (bodyType) {
   test(`pipelining HEAD busy ${bodyType}`, async (t) => {
     t = tspl(t, { plan: 7 })
 
-    const server = createServer()
+    const server = createServer({ joinDuplicateHeaders: true })
     server.on('request', (req, res) => {
       res.end('asd')
     })
@@ -561,7 +561,7 @@ test('pipelining empty pipeline before reset', async (t) => {
   t = tspl(t, { plan: 8 })
 
   let c = 0
-  const server = createServer()
+  const server = createServer({ joinDuplicateHeaders: true })
   server.on('request', (req, res) => {
     if (c++ === 0) {
       res.end('asd')
@@ -624,7 +624,7 @@ function pipeliningIdempotentBusy (bodyType) {
   test(`pipelining idempotent busy ${bodyType}`, async (t) => {
     t = tspl(t, { plan: 12 })
 
-    const server = createServer()
+    const server = createServer({ joinDuplicateHeaders: true })
     server.on('request', (req, res) => {
       res.end('asd')
     })
@@ -728,7 +728,7 @@ pipeliningIdempotentBusy(consts.ASYNC_ITERATOR)
 test('pipelining blocked', async (t) => {
   t = tspl(t, { plan: 6 })
 
-  const server = createServer()
+  const server = createServer({ joinDuplicateHeaders: true })
 
   let blocking = true
   let count = 0
