@@ -8,7 +8,7 @@ const assert = require('node:assert')
 const { closeServerAsPromise } = require('../utils/node-http')
 
 test('error reason is forwarded - issue #2171', { skip: !AbortSignal.timeout }, async (t) => {
-  const server = createServer(() => {}).listen(0)
+  const server = createServer({ joinDuplicateHeaders: true }, () => {}).listen(0)
 
   t.after(closeServerAsPromise(server))
   await once(server, 'listening')
