@@ -12,7 +12,12 @@ const {
   HTTPS_NOTSAMESITE_ORIGIN,
 } = get_host_info();
 
-const description = 'Default "deferred-fetch" permissions policy ["self"]';
+// https://whatpr.org/fetch/1647.html#dom-permissionspolicy-deferred-fetch
+const deferredFetchPolicy =
+    'Default "deferred-fetch" permissions policy ["self"]';
+// https://whatpr.org/fetch/1647.html#dom-permissionspolicy-deferred-fetch-minimal
+const deferredFetchMinimalPolicy =
+    'Default "deferred-fetch-minimal" permissions policy ["*"]';
 
 parallelPromiseTest(async _ => {
   const uuid = token();
@@ -22,17 +27,17 @@ parallelPromiseTest(async _ => {
   fetchLater(url, {activateAfter: 0});
 
   await expectBeacon(uuid, {count: 1});
-}, `${description} allows fetchLater() in the top-level document.`);
+}, `${deferredFetchPolicy} allows fetchLater() in the top-level document.`);
 
 async_test(t => {
   test_feature_availability(
       'fetchLater()', t, getDeferredFetchPolicyInIframeHelperUrl(HTTPS_ORIGIN),
       expect_feature_available_default);
-}, `${description} allows fetchLater() in the same-origin iframe.`);
+}, `${deferredFetchPolicy} allows fetchLater() in the same-origin iframe.`);
 
 async_test(t => {
   test_feature_availability(
       'fetchLater()', t,
       getDeferredFetchPolicyInIframeHelperUrl(HTTPS_NOTSAMESITE_ORIGIN),
       expect_feature_available_default);
-}, `${description} allows fetchLater() in the cross-origin iframe.`);
+}, `${deferredFetchMinimalPolicy} allows fetchLater() in the cross-origin iframe.`);
