@@ -5,10 +5,9 @@ const { test, after } = require('node:test')
 const { createSecureServer } = require('node:http2')
 const { createReadStream, readFileSync } = require('node:fs')
 const { once } = require('node:events')
-const { Blob } = require('node:buffer')
 const { Writable, pipeline, PassThrough, Readable } = require('node:stream')
 
-const pem = require('https-pem')
+const pem = require('@metcoder95/https-pem')
 
 const { Client, Agent, FormData } = require('..')
 
@@ -16,7 +15,7 @@ const isGreaterThanv20 = process.versions.node.split('.').map(Number)[0] >= 20
 
 test('Should support H2 connection', async t => {
   const body = []
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers, _flags, rawHeaders) => {
     t.strictEqual(headers['x-my-header'], 'foo')
@@ -63,7 +62,7 @@ test('Should support H2 connection', async t => {
 })
 
 test('Should support H2 connection(multiple requests)', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', async (stream, headers, _flags, rawHeaders) => {
     t.strictEqual(headers['x-my-header'], 'foo')
@@ -122,11 +121,12 @@ test('Should support H2 connection(multiple requests)', async t => {
 
 test('Should support H2 connection (headers as array)', async t => {
   const body = []
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
-    t.strictEqual(headers['x-my-header'], 'foo')
-    t.strictEqual(headers['x-my-drink'], 'coffee,tea')
+    t.strictEqual(headers['x-my-header'], 'foo, bar')
+    t.strictEqual(headers['x-my-drink'], 'coffee, tea, water')
+    t.strictEqual(headers['x-other'], 'value')
     t.strictEqual(headers[':method'], 'GET')
     stream.respond({
       'content-type': 'text/plain; charset=utf-8',
@@ -146,14 +146,20 @@ test('Should support H2 connection (headers as array)', async t => {
     allowH2: true
   })
 
-  t = tspl(t, { plan: 7 })
+  t = tspl(t, { plan: 8 })
   after(() => server.close())
   after(() => client.close())
 
   const response = await client.request({
     path: '/',
     method: 'GET',
-    headers: ['x-my-header', 'foo', 'x-my-drink', ['coffee', 'tea']]
+    headers: [
+      'x-my-header', 'foo',
+      'x-my-drink', ['coffee', 'tea'],
+      'x-my-drink', 'water',
+      'X-My-Header', 'bar',
+      'x-other', 'value'
+    ]
   })
 
   response.body.on('data', chunk => {
@@ -168,7 +174,7 @@ test('Should support H2 connection (headers as array)', async t => {
 })
 
 test('Should support H2 connection(POST Buffer)', async t => {
-  const server = createSecureServer({ ...pem, allowHTTP1: false })
+  const server = createSecureServer({ ...await pem.generate({ opts: { keySize: 2048 } }), allowHTTP1: false })
 
   server.on('stream', async (stream, headers, _flags, rawHeaders) => {
     t.strictEqual(headers[':method'], 'POST')
@@ -273,7 +279,7 @@ test(
 
     const server = createSecureServer(
       {
-        ...pem,
+        ...await pem.generate({ opts: { keySize: 2048 } }),
         allowHTTP1: false,
         ALPNProtocols: ['http/1.1']
       },
@@ -313,7 +319,7 @@ test(
   async t => {
     const server = createSecureServer(
       {
-        ...pem,
+        ...await pem.generate({ opts: { keySize: 2048 } }),
         allowHTTP1: false,
         ALPNProtocols: ['http/1.1']
       },
@@ -348,7 +354,7 @@ test(
 
 test('Should handle h2 continue', async t => {
   const requestBody = []
-  const server = createSecureServer(pem, () => {})
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }), () => {})
   const responseBody = []
 
   server.on('checkContinue', (request, response) => {
@@ -404,7 +410,7 @@ test('Should handle h2 continue', async t => {
 })
 
 test('Dispatcher#Stream', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = 'hello from client!'
   const bufs = []
   let requestBody = ''
@@ -455,7 +461,7 @@ test('Dispatcher#Stream', async t => {
 })
 
 test('Dispatcher#Pipeline', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = 'hello from client!'
   const bufs = []
   let requestBody = ''
@@ -517,7 +523,7 @@ test('Dispatcher#Pipeline', async t => {
 })
 
 test('Dispatcher#Connect', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = 'hello from client!'
   let requestBody = ''
 
@@ -574,7 +580,7 @@ test('Dispatcher#Connect', async t => {
 })
 
 test('Dispatcher#Upgrade', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', async (stream, headers) => {
     stream.end()
@@ -605,7 +611,7 @@ test('Dispatcher#Upgrade', async t => {
 
 test('Dispatcher#destroy', async t => {
   const promises = []
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
     setTimeout(stream.end.bind(stream), 1500)
@@ -675,7 +681,7 @@ test('Dispatcher#destroy', async t => {
 })
 
 test('Should handle h2 request without body', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = ''
   const requestChunks = []
   const responseBody = []
@@ -734,7 +740,7 @@ test('Should handle h2 request without body', async t => {
 })
 
 test('Should handle h2 request with body (string or buffer) - dispatch', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = 'hello from client!'
   const response = []
   const requestBody = []
@@ -812,7 +818,7 @@ test('Should handle h2 request with body (string or buffer) - dispatch', async t
 })
 
 test('Should handle h2 request with body (stream)', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = readFileSync(__filename, 'utf-8')
   const stream = createReadStream(__filename)
   const requestChunks = []
@@ -872,7 +878,7 @@ test('Should handle h2 request with body (stream)', async t => {
 })
 
 test('Should handle h2 request with body (iterable)', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = 'hello'
   const requestChunks = []
   const responseBody = []
@@ -941,7 +947,7 @@ test('Should handle h2 request with body (iterable)', async t => {
 })
 
 test('Should handle h2 request with body (Blob)', { skip: !Blob }, async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const expectedBody = 'asd'
   const requestChunks = []
   const responseBody = []
@@ -1006,7 +1012,7 @@ test(
   'Should handle h2 request with body (Blob:ArrayBuffer)',
   { skip: !Blob },
   async t => {
-    const server = createSecureServer(pem)
+    const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
     const expectedBody = 'hello'
     const requestChunks = []
     const responseBody = []
@@ -1071,7 +1077,7 @@ test(
 
 test('Agent should support H2 connection', async t => {
   const body = []
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
     t.strictEqual(headers['x-my-header'], 'foo')
@@ -1121,7 +1127,7 @@ test('Agent should support H2 connection', async t => {
 test('Should provide pseudo-headers in proper order', async t => {
   t = tspl(t, { plan: 2 })
 
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   server.on('stream', (stream, _headers, _flags, rawHeaders) => {
     t.deepStrictEqual(rawHeaders, [
       ':authority',
@@ -1167,7 +1173,7 @@ test('Should provide pseudo-headers in proper order', async t => {
 })
 
 test('The h2 pseudo-headers is not included in the headers', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
     stream.respond({
@@ -1202,7 +1208,7 @@ test('The h2 pseudo-headers is not included in the headers', async t => {
 })
 
 test('Should throw informational error on half-closed streams (remote)', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
     stream.destroy()
@@ -1245,7 +1251,7 @@ test('Should throw informational error on half-closed streams (remote)', async t
 })
 
 test('#2364 - Concurrent aborts', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers, _flags, rawHeaders) => {
     setTimeout(() => {
@@ -1343,7 +1349,7 @@ test('#2364 - Concurrent aborts', async t => {
 })
 
 test('#2364 - Concurrent aborts (2nd variant)', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   let counter = 0
 
   server.on('stream', (stream, headers, _flags, rawHeaders) => {
@@ -1461,7 +1467,7 @@ test('#2364 - Concurrent aborts (2nd variant)', async t => {
 })
 
 test('#3046 - GOAWAY Frame', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
     setTimeout(() => {
@@ -1492,7 +1498,7 @@ test('#3046 - GOAWAY Frame', async t => {
     allowH2: true
   })
 
-  t = tspl(t, { plan: 7 })
+  t = tspl(t, { plan: 10 })
   after(() => client.close())
   after(() => server.close())
 
@@ -1502,6 +1508,12 @@ test('#3046 - GOAWAY Frame', async t => {
     t.strictEqual(err.message, 'HTTP/2: "GOAWAY" frame received with code 0')
   })
 
+  client.on('connectionError', (url, disconnectClient, err) => {
+    t.ok(url instanceof URL)
+    t.deepStrictEqual(disconnectClient, [client])
+    t.strictEqual(err.message, 'HTTP/2: "GOAWAY" frame received with code 0')
+  })
+
   const response = await client.request({
     path: '/',
     method: 'GET',
@@ -1514,7 +1526,7 @@ test('#3046 - GOAWAY Frame', async t => {
   t.strictEqual(response.headers['x-custom-h2'], 'hello')
   t.strictEqual(response.statusCode, 200)
 
-  t.rejects(response.body.text(), {
+  await t.rejects(response.body.text(), {
     message: 'HTTP/2: "GOAWAY" frame received with code 0',
     code: 'UND_ERR_SOCKET'
   })
@@ -1523,7 +1535,7 @@ test('#3046 - GOAWAY Frame', async t => {
 })
 
 test('#3671 - Graceful close', async (t) => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', (stream, headers) => {
     setTimeout(() => {
@@ -1569,7 +1581,7 @@ test('#3671 - Graceful close', async (t) => {
 })
 
 test('#3753 - Handle GOAWAY Gracefully', async (t) => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   let counter = 0
   let session = null
 
@@ -1646,7 +1658,7 @@ test('#3753 - Handle GOAWAY Gracefully', async (t) => {
 test('#3803 - sending FormData bodies works', async (t) => {
   const assert = tspl(t, { plan: 4 })
 
-  const server = createSecureServer(pem).listen(0)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } })).listen(0)
   server.on('stream', async (stream, headers) => {
     const contentLength = Number(headers['content-length'])
 
@@ -1695,7 +1707,7 @@ test('#3803 - sending FormData bodies works', async (t) => {
 })
 
 test('Should handle http2 stream timeout', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
   const stream = createReadStream(__filename)
 
   server.on('stream', async (stream, headers) => {
@@ -1735,13 +1747,15 @@ test('Should handle http2 stream timeout', async t => {
     body: stream
   })
 
-  t.rejects(res.body.text(), {
+  await t.rejects(res.body.text(), {
     message: 'HTTP/2: "stream timeout after 50"'
   })
+
+  await t.completed
 })
 
 test('Should handle http2 trailers', async t => {
-  const server = createSecureServer(pem)
+  const server = createSecureServer(await pem.generate({ opts: { keySize: 2048 } }))
 
   server.on('stream', async (stream, headers) => {
     stream.respond({
