@@ -10,7 +10,7 @@ const { Client, interceptors, cacheStores: { MemoryCacheStore } } = require('../
 describe('Cache Interceptor', () => {
   test('caches request', async () => {
     let requestsToOrigin = 0
-    const server = createServer((_, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
       requestsToOrigin++
       res.setHeader('cache-control', 's-maxage=10')
       res.end('asd')
@@ -53,7 +53,7 @@ describe('Cache Interceptor', () => {
 
   test('vary directives used to decide which response to use', async () => {
     let requestsToOrigin = 0
-    const server = createServer((req, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
       requestsToOrigin++
       res.setHeader('cache-control', 's-maxage=10')
       res.setHeader('vary', 'a')
@@ -128,6 +128,264 @@ describe('Cache Interceptor', () => {
     }
   })
 
+  test('revalidates reponses with no-cache directive, regardless of cacheByDefault', async () => {
+    let requestCount = 0
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
+      ++requestCount
+      res.setHeader('Vary', 'Accept-Encoding')
+      res.setHeader('cache-control', 'no-cache')
+      res.end(`Request count: ${requestCount}`)
+    }).listen(0)
+
+    after(async () => {
+      server.close()
+
+      await once(server, 'close')
+    })
+
+    await once(server, 'listening')
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache({
+        cacheByDefault: 1000
+      }))
+
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/'
+    }
+
+    const res1 = await client.request(request)
+    const body1 = await res1.body.text()
+    strictEqual(body1, 'Request count: 1')
+    strictEqual(requestCount, 1)
+
+    const res2 = await client.request(request)
+    const body2 = await res2.body.text()
+    strictEqual(body2, 'Request count: 2')
+    strictEqual(requestCount, 2)
+  })
+
+  test('expires caching', async () => {
+    const clock = FakeTimers.install({
+      shouldClearNativeTimers: true
+    })
+
+    let requestsToOrigin = 0
+    let serverError
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
+      const now = new Date()
+      now.setSeconds(now.getSeconds() + 1)
+      res.setHeader('date', 0)
+      res.setHeader('expires', now.toGMTString())
+      requestsToOrigin++
+      res.end('asd')
+    }).listen(0)
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache())
+
+    after(async () => {
+      server.close()
+      await client.close()
+      clock.uninstall()
+    })
+
+    await once(server, 'listening')
+
+    strictEqual(requestsToOrigin, 0)
+
+    /**
+     * @type {import('../../types/dispatcher').default.RequestOptions}
+     */
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/'
+    }
+
+    // Send initial request. This should reach the origin
+    {
+      const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    // This is cached
+    {
+      const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    clock.tick(1500)
+
+    // Response is now stale, the origin should get a request
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    // Response is now cached, the origin should not get a request
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+  })
+
+  test('expires caching with Etag', async () => {
+    const clock = FakeTimers.install({
+      shouldClearNativeTimers: true
+    })
+
+    let requestsToOrigin = 0
+    let serverError
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
+      const now = new Date()
+      now.setSeconds(now.getSeconds() + 1)
+      res.setHeader('date', 0)
+      res.setHeader('expires', now.toGMTString())
+      res.setHeader('etag', 'asd123')
+      requestsToOrigin++
+      res.end('asd')
+    }).listen(0)
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache())
+
+    after(async () => {
+      server.close()
+      await client.close()
+      clock.uninstall()
+    })
+
+    await once(server, 'listening')
+
+    strictEqual(requestsToOrigin, 0)
+
+    /**
+     * @type {import('../../types/dispatcher').default.RequestOptions}
+     */
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/'
+    }
+
+    // Send initial request. This should reach the origin
+    {
+      const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    // This is cached
+    {
+      const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    clock.tick(1500)
+
+    // Response is now stale, the origin should get a request
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    // Response is now cached, the origin should not get a request
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+  })
+
+  test('max-age caching', async () => {
+    const clock = FakeTimers.install({
+      shouldClearNativeTimers: true
+    })
+
+    let requestsToOrigin = 0
+    let serverError
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
+      res.setHeader('date', 0)
+      res.setHeader('cache-control', 's-maxage=1')
+      requestsToOrigin++
+      res.end('asd')
+    }).listen(0)
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache())
+
+    after(async () => {
+      server.close()
+      await client.close()
+      clock.uninstall()
+    })
+
+    await once(server, 'listening')
+
+    strictEqual(requestsToOrigin, 0)
+
+    /**
+     * @type {import('../../types/dispatcher').default.RequestOptions}
+     */
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/'
+    }
+
+    // Send initial request. This should reach the origin
+    {
+      const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    clock.tick(1500)
+
+    // Response is now stale, the origin should get a request
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
+    // Response is now cached, the origin should not get a request
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+  })
+
   test('stale responses are revalidated before deleteAt (if-modified-since)', async () => {
     const clock = FakeTimers.install({
       shouldClearNativeTimers: true
@@ -135,22 +393,30 @@ describe('Cache Interceptor', () => {
 
     let requestsToOrigin = 0
     let revalidationRequests = 0
-    const server = createServer((req, res) => {
+    let serverError
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
       res.setHeader('date', 0)
       res.setHeader('cache-control', 's-maxage=1, stale-while-revalidate=10')
 
-      if (req.headers['if-modified-since']) {
-        revalidationRequests++
+      try {
+        if (req.headers['if-modified-since']) {
+          equal(req.headers['if-modified-since'].length, 29)
 
-        if (revalidationRequests === 2) {
-          res.end('updated')
+          revalidationRequests++
+
+          if (revalidationRequests === 3) {
+            res.end('updated')
+          } else {
+            res.statusCode = 304
+            res.end()
+          }
         } else {
-          res.statusCode = 304
-          res.end()
+          requestsToOrigin++
+          res.end('asd')
         }
-      } else {
-        requestsToOrigin++
-        res.end('asd')
+      } catch (err) {
+        serverError = err
+        res.end()
       }
     }).listen(0)
 
@@ -188,6 +454,10 @@ describe('Cache Interceptor', () => {
     // Send initial request. This should reach the origin
     {
       const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
       equal(requestsToOrigin, 1)
       equal(revalidationRequests, 0)
       strictEqual(await res.body.text(), 'asd')
@@ -198,16 +468,42 @@ describe('Cache Interceptor', () => {
     // Response is now stale, the origin should get a revalidation request
     {
       const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
       equal(requestsToOrigin, 1)
       equal(revalidationRequests, 1)
       strictEqual(await res.body.text(), 'asd')
     }
 
+    // Response is still stale, extra header should be overwritten, and the
+    // origin should get a revalidation request
+    {
+      const res = await client.request({
+        ...request,
+        headers: {
+          'if-modified-SINCE': 'Thu, 01 Jan 1970 00:00:00 GMT'
+        }
+      })
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      equal(revalidationRequests, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
     // Response is still stale, but revalidation should fail now.
     {
       const res = await client.request(request)
+      if (serverError) {
+        throw serverError
+      }
+
       equal(requestsToOrigin, 1)
-      equal(revalidationRequests, 2)
+      equal(revalidationRequests, 3)
       strictEqual(await res.body.text(), 'updated')
     }
   })
@@ -220,7 +516,7 @@ describe('Cache Interceptor', () => {
     let requestsToOrigin = 0
     let revalidationRequests = 0
     let serverError
-    const server = createServer((req, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
       res.setHeader('date', 0)
       res.setHeader('cache-control', 's-maxage=1, stale-while-revalidate=10')
 
@@ -230,7 +526,7 @@ describe('Cache Interceptor', () => {
 
           equal(req.headers['if-none-match'], '"asd123"')
 
-          if (revalidationRequests === 2) {
+          if (revalidationRequests === 3) {
             res.end('updated')
           } else {
             res.statusCode = 304
@@ -296,6 +592,24 @@ describe('Cache Interceptor', () => {
       strictEqual(await res.body.text(), 'asd')
     }
 
+    // Response is still stale, extra headers should be overwritten, and the
+    // origin should get a revalidation request
+    {
+      const res = await client.request({
+        ...request,
+        headers: {
+          'if-NONE-match': '"nonsense-etag"'
+        }
+      })
+      if (serverError) {
+        throw serverError
+      }
+
+      equal(requestsToOrigin, 1)
+      equal(revalidationRequests, 2)
+      strictEqual(await res.body.text(), 'asd')
+    }
+
     // Response is still stale, but revalidation should fail now.
     {
       const res = await client.request(request)
@@ -304,7 +618,7 @@ describe('Cache Interceptor', () => {
       }
 
       equal(requestsToOrigin, 1)
-      equal(revalidationRequests, 2)
+      equal(revalidationRequests, 3)
       strictEqual(await res.body.text(), 'updated')
     }
   })
@@ -317,7 +631,7 @@ describe('Cache Interceptor', () => {
     let requestsToOrigin = 0
     let revalidationRequests = 0
     let serverError
-    const server = createServer((req, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
       res.setHeader('date', 0)
       res.setHeader('cache-control', 's-maxage=1, stale-while-revalidate=10')
 
@@ -327,13 +641,13 @@ describe('Cache Interceptor', () => {
         if (ifNoneMatch) {
           revalidationRequests++
           notEqual(req.headers.a, undefined)
-          notEqual(req.headers.b, undefined)
+          notEqual(req.headers['b-mixed-case'], undefined)
 
           res.statusCode = 304
           res.end()
         } else {
           requestsToOrigin++
-          res.setHeader('vary', 'a, b')
+          res.setHeader('vary', 'a, B-MIXED-CASe')
           res.setHeader('etag', '"asd"')
           res.end('asd')
         }
@@ -360,15 +674,17 @@ describe('Cache Interceptor', () => {
     const request = {
       origin: 'localhost',
       path: '/',
-      method: 'GET',
-      headers: {
-        a: 'asd',
-        b: 'asd'
-      }
+      method: 'GET'
     }
 
     {
-      const response = await client.request(request)
+      const response = await client.request({
+        ...request,
+        headers: {
+          a: 'asd',
+          'b-Mixed-case': 'asd'
+        }
+      })
       if (serverError) {
         throw serverError
       }
@@ -380,7 +696,13 @@ describe('Cache Interceptor', () => {
     clock.tick(1500)
 
     {
-      const response = await client.request(request)
+      const response = await client.request({
+        ...request,
+        headers: {
+          a: 'asd',
+          'B-mixed-CASE': 'asd'
+        }
+      })
       if (serverError) {
         throw serverError
       }
@@ -392,7 +714,7 @@ describe('Cache Interceptor', () => {
   })
 
   test('unsafe methods cause resource to be purged from cache', async () => {
-    const server = createServer((_, res) => res.end('asd')).listen(0)
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => res.end('asd')).listen(0)
 
     after(() => server.close())
     await once(server, 'listening')
@@ -441,7 +763,7 @@ describe('Cache Interceptor', () => {
   })
 
   test('unsafe methods aren\'t cached', async () => {
-    const server = createServer((_, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
       res.setHeader('cache-control', 'public, s-maxage=1')
       res.end('')
     }).listen(0)
@@ -485,7 +807,7 @@ describe('Cache Interceptor', () => {
     ]
 
     let requestToOrigin = 0
-    const server = createServer((_, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
       requestToOrigin++
       res.setHeader('cache-control', 's-maxage=10, no-cache=should-be-stripped')
       res.setHeader('should-not-be-stripped', 'asd')
@@ -540,7 +862,7 @@ describe('Cache Interceptor', () => {
 
   test('cacheByDefault', async () => {
     let requestsToOrigin = 0
-    const server = createServer((_, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
       requestsToOrigin++
       res.end('asd')
     }).listen(0)
@@ -583,7 +905,7 @@ describe('Cache Interceptor', () => {
     })
 
     let requestsToOrigin = 0
-    const server = createServer((_, res) => {
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
       res.setHeader('date', 0)
 
       requestsToOrigin++
@@ -665,7 +987,7 @@ describe('Cache Interceptor', () => {
       })
 
       let requestsToOrigin = 0
-      const server = createServer((_, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
         requestsToOrigin++
         res.setHeader('date', 0)
         res.setHeader('cache-control', 'public, s-maxage=100')
@@ -727,7 +1049,7 @@ describe('Cache Interceptor', () => {
 
       let requestsToOrigin = 0
       let revalidationRequests = 0
-      const server = createServer((req, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
         res.setHeader('date', 0)
         res.setHeader('cache-control', 'public, s-maxage=1, stale-while-revalidate=10')
 
@@ -796,7 +1118,7 @@ describe('Cache Interceptor', () => {
       })
 
       let requestsToOrigin = 0
-      const server = createServer((_, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
         requestsToOrigin++
         res.setHeader('date', 0)
         res.setHeader('cache-control', 'public, s-maxage=10')
@@ -855,7 +1177,7 @@ describe('Cache Interceptor', () => {
     test('no-cache', async () => {
       let requestsToOrigin = 0
       let revalidationRequests = 0
-      const server = createServer((req, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (req, res) => {
         if (req.headers['if-modified-since']) {
           revalidationRequests++
           res.statusCode = 304
@@ -914,7 +1236,7 @@ describe('Cache Interceptor', () => {
     })
 
     test('no-store', async () => {
-      const server = createServer((_, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
         res.setHeader('cache-control', 'public, s-maxage=100')
         res.end('asd')
       }).listen(0)
@@ -946,7 +1268,7 @@ describe('Cache Interceptor', () => {
 
     test('only-if-cached', async () => {
       let requestsToOrigin = 0
-      const server = createServer((_, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
         res.setHeader('cache-control', 'public, s-maxage=100')
         res.end('asd')
         requestsToOrigin++
@@ -1014,7 +1336,7 @@ describe('Cache Interceptor', () => {
       })
 
       let requestsToOrigin = 0
-      const server = createServer((_, res) => {
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
         res.setHeader('date', 0)
 
         requestsToOrigin++
@@ -1114,4 +1436,265 @@ describe('Cache Interceptor', () => {
       }
     })
   })
+
+  // Partial list.
+  const cacheableStatusCodes = [
+    { code: 204, body: '' },
+    { code: 302, body: 'Found' },
+    { code: 307, body: 'Temporary Redirect' },
+    { code: 404, body: 'Not Found' },
+    { code: 410, body: 'Gone' }
+  ]
+
+  for (const { code, body } of cacheableStatusCodes) {
+    test(`caches ${code} response with cache headers`, async () => {
+      let requestsToOrigin = 0
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
+        requestsToOrigin++
+        res.statusCode = code
+        res.setHeader('cache-control', 'public, max-age=60')
+        res.end(body)
+      }).listen(0)
+
+      const client = new Client(`http://localhost:${server.address().port}`)
+        .compose(interceptors.cache())
+
+      after(async () => {
+        server.close()
+        await client.close()
+      })
+
+      await once(server, 'listening')
+
+      equal(requestsToOrigin, 0)
+
+      const request = {
+        origin: 'localhost',
+        method: 'GET',
+        path: '/'
+      }
+
+      // First request should hit the origin
+      {
+        const res = await client.request(request)
+        equal(requestsToOrigin, 1)
+        equal(res.statusCode, code)
+        strictEqual(await res.body.text(), body)
+      }
+
+      // Second request should be served from cache
+      {
+        const res = await client.request(request)
+        equal(requestsToOrigin, 1) // Should still be 1 (cached)
+        equal(res.statusCode, code)
+        strictEqual(await res.body.text(), body)
+      }
+    })
+  }
+
+  // Partial list.
+  const nonHeuristicallyCacheableStatusCodes = [
+    { code: 201, body: 'Created' },
+    { code: 307, body: 'Temporary Redirect' },
+    { code: 418, body: 'I am a teapot' }
+  ]
+
+  for (const { code, body } of nonHeuristicallyCacheableStatusCodes) {
+    test(`does not cache non-heuristically cacheable status ${code} without explicit directive`, async () => {
+      let requestsToOrigin = 0
+      const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
+        requestsToOrigin++
+        res.statusCode = code
+        // By default the response may have a date and last-modified header set to 'now',
+        // causing the cache to compute a 0 heuristic expiry, causing the test to not ascertain
+        // it is really not cached.
+        res.setHeader('date', '')
+        res.end(body)
+      }).listen(0)
+
+      const client = new Client(`http://localhost:${server.address().port}`)
+        .compose(interceptors.cache({ cacheByDefault: 60 }))
+
+      after(async () => {
+        server.close()
+        await client.close()
+      })
+
+      await once(server, 'listening')
+
+      equal(requestsToOrigin, 0)
+
+      const request = {
+        origin: 'localhost',
+        method: 'GET',
+        path: '/'
+      }
+
+      // First request should hit the origin
+      {
+        const res = await client.request(request)
+        equal(requestsToOrigin, 1)
+        equal(res.statusCode, code)
+        strictEqual(await res.body.text(), body)
+      }
+
+      // Second request should also hit the origin (not cached)
+      {
+        const res = await client.request(request)
+        equal(requestsToOrigin, 2) // Should be 2 (not cached)
+        equal(res.statusCode, code)
+        strictEqual(await res.body.text(), body)
+      }
+    })
+  }
+
+  test('discriminates caching of range requests, or does not cache them', async () => {
+    let requestsToOrigin = 0
+    const body = 'Fake range request response'
+    const code = 206
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
+      requestsToOrigin++
+      res.statusCode = code
+      res.setHeader('cache-control', 'public, max-age=60')
+      res.end(body)
+    }).listen(0)
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache())
+
+    after(async () => {
+      server.close()
+      await client.close()
+    })
+
+    await once(server, 'listening')
+
+    equal(requestsToOrigin, 0)
+
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/',
+      headers: {
+        range: 'bytes=10-'
+      }
+    }
+
+    // First request should hit the origin
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 1)
+      equal(res.statusCode, code)
+      strictEqual(await res.body.text(), body)
+    }
+
+    // Second request with different range should hit the origin too
+    request.headers.range = 'bytes=5-'
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      equal(res.statusCode, code)
+      strictEqual(await res.body.text(), body)
+    }
+  })
+
+  test('discriminates caching of conditionnal requests (if-none-match), or does not cache them', async () => {
+    let requestsToOrigin = 0
+    const body = ''
+    const code = 304
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
+      requestsToOrigin++
+      res.statusCode = code
+      res.setHeader('cache-control', 'public, max-age=60')
+      res.end(body)
+    }).listen(0)
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache())
+
+    after(async () => {
+      server.close()
+      await client.close()
+    })
+
+    await once(server, 'listening')
+
+    equal(requestsToOrigin, 0)
+
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/',
+      headers: {
+        'if-none-match': 'some-etag'
+      }
+    }
+
+    // First request should hit the origin
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 1)
+      equal(res.statusCode, code)
+      strictEqual(await res.body.text(), body)
+    }
+
+    // Second request with different etag should hit the origin too
+    request.headers['if-none-match'] = 'another-etag'
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      equal(res.statusCode, code)
+      strictEqual(await res.body.text(), body)
+    }
+  })
+
+  test('discriminates caching of conditionnal requests (if-modified-since), or does not cache them', async () => {
+    let requestsToOrigin = 0
+    const body = ''
+    const code = 304
+    const server = createServer({ joinDuplicateHeaders: true }, (_, res) => {
+      requestsToOrigin++
+      res.statusCode = code
+      res.setHeader('cache-control', 'public, max-age=60')
+      res.end(body)
+    }).listen(0)
+
+    const client = new Client(`http://localhost:${server.address().port}`)
+      .compose(interceptors.cache())
+
+    after(async () => {
+      server.close()
+      await client.close()
+    })
+
+    await once(server, 'listening')
+
+    equal(requestsToOrigin, 0)
+
+    const request = {
+      origin: 'localhost',
+      method: 'GET',
+      path: '/',
+      headers: {
+        'if-modified-since': new Date().toUTCString()
+      }
+    }
+
+    // First request should hit the origin
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 1)
+      equal(res.statusCode, code)
+      strictEqual(await res.body.text(), body)
+    }
+
+    // Second request with different since should hit the origin too
+    request.headers['if-modified-since'] = new Date(0).toUTCString()
+    {
+      const res = await client.request(request)
+      equal(requestsToOrigin, 2)
+      equal(res.statusCode, code)
+      strictEqual(await res.body.text(), body)
+    }
+  })
 })
