@@ -2,10 +2,16 @@ import { expectAssignable } from 'tsd'
 import BodyReadable from '../../types/readable'
 import { Blob } from 'buffer'
 
-expectAssignable<BodyReadable>(new BodyReadable())
+expectAssignable<BodyReadable>(new BodyReadable({
+  abort: () => null,
+  resume: () => null
+}))
 
 {
-  const readable = new BodyReadable()
+  const readable = new BodyReadable({
+    abort: () => null,
+    resume: () => null
+  })
 
   // dump
   expectAssignable<Promise<void>>(readable.dump())
@@ -20,6 +26,9 @@ expectAssignable<BodyReadable>(new BodyReadable())
   // blob
   expectAssignable<Promise<Blob>>(readable.blob())
 
+  // bytes
+  expectAssignable<Promise<Uint8Array>>(readable.bytes())
+
   // arrayBuffer
   expectAssignable<Promise<ArrayBuffer>>(readable.arrayBuffer())
 
